;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PICmicro PIC16F877. This file contains the basic code      *
;   building blocks to build upon.                                    *  
;                                                                     *
;   If interrupts are not used all code presented between the ORG     *
;   0x004 directive and the label main can be removed. In addition    *
;   the variable assignments for 'w_temp' and 'status_temp' can       *
;   be removed.                                                       *                         
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PICmicro data sheet for additional        *
;   information on the instruction set.                               *
;                                                                     *
;   Template file assembled with MPLAB V4.00 and MPASM V2.20.00.      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	   rweeprom.asm                                       *
;    Date:         03/08/2544                                         *
;    File Version: 1.00                                               *
;                                                                     *
;    Author:       PITAK TISUTHIWONGSE                                *
;    Company:                                                         *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;     OSC = 4 MHZ                                                     *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************


	list      p=16f877            ; list directive to define processor
	#include <p16f877.inc>        ; processor specific variable definitions
	
;	__CONFIG _CP_OFF & _WDT_ON & _BODEN_ON & _PWRTE_ON & _RC_OSC & _WRT_ENABLE_ON & _LVP_ON & _DEBUG_OFF & _CPD_OFF 

; '__CONFIG' directive is used to embed configuration data within .asm file.
; The lables following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.

;***** VARIABLE DEFINITIONS
My_EEDATA	EQU	0x20		; variable keep EEPROM DATA
My_EEADDR	EQU	0x21		; variable keep EEPROM ADDRESS
TempCount	EQU	0x22		; Temp count

w_temp        	EQU     0x70    	; variable used for context saving 
status_temp   	EQU     0x71        	; variable used for context saving

;**********************************************************************
		ORG     0x000             ; processor reset vector
		nop
		clrf    PCLATH            ; ensure page bits are cleared
  		goto    main              ; go to beginning of program


		ORG     0x004             ; interrupt vector location
		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register
		;
		bcf	PIR1,TMR1IF
		call	Timer1_Intr	  ; Call Timer1 Interrupt
		;
		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt
;-----------------------------------------------------------------------------
;			 MAIN ROUTINE START HERE
;-----------------------------------------------------------------------------
	        ORG     0x0020
main:
;-----------------------------------------------------------------------------
; NAME		: Init_M05_SYSTEM
; PURPOSE	: Initialize program
;-----------------------------------------------------------------------------
Init_program:
;-----------------------------------------------------------------------------
; If Fosc = 18.432 MHz Prescaler = 1:1 then watch dog timeout = 18 mS
;-----------------------------------------------------------------------------
	call    Init_Timer		; Initialize timer register
        call    Init_Ports		; Initialize PORT of M05
	movlw	b'11000000'		
	movwf	INTCON
	bsf     STATUS,5		; Select bank 1
	bcf	STATUS,6
	movlw	b'00000001'		; Enable Timer1 Interrupt
	movwf	PIE1
	bcf     STATUS,5		; Select bank 0
	bcf	STATUS,6
	clrf	PORTA
	movlw	h'FF'			; Drive LOW set all PORTD HIGH
	movwf	PORTD
;-----------------------------------------------------------------------------
; Program loop start here
;-----------------------------------------------------------------------------
StartLoop:
	movlw	h'10'			; Start at address h'00'
	movwf	My_EEADDR
	movlw	a'H'			; Write Hello to EEPROM
	movwf	My_EEDATA		
	call	EE_WRITE_BYTE
	incf	My_EEADDR,f
	movlw	a'e'
	movwf	My_EEDATA
	call	EE_WRITE_BYTE
	incf	My_EEADDR,f
	movlw	a'l'
	movwf	My_EEDATA
	call	EE_WRITE_BYTE
	incf	My_EEADDR,f
	movlw	a'l'
	movwf	My_EEDATA
	call	EE_WRITE_BYTE
	incf	My_EEADDR,f
	movlw	a'o'
	movwf	My_EEDATA
	call	EE_WRITE_BYTE
ReadBack:
	movlw	h'10'			; Start to read at address h'10' of EEPROM
	movwf	My_EEADDR
	movlw	h'30'			; Keep to address 30h-34h of RAM
	movwf	FSR
	movlw	h'5'			; Read 5 bytes
	movwf	TempCount
ReadEEP_Seq:				; Start Read
	call	EE_READ_BYTE		
	movfw	My_EEDATA
	movwf	INDF
	incf	FSR,f
	incf	My_EEADDR,f
	decfsz  TempCount,f		; Decrease counter
	goto	ReadEEP_Seq
Endless_Loop:
	goto	Endless_Loop
;-----------------------------------------------------------------------------
; Timer1 Interrupt
;-----------------------------------------------------------------------------
Timer1_Intr:
	nop					; No any thing to do now
	return
;-----------------------------------------------------------------------------
; EE_WRITE_BYTE	- Routine to write a byte to EEPROM
; DATA IN - My_EEDATA	<- data want to write
;	  - My_EEADDR   <- address want to write
;-----------------------------------------------------------------------------
EE_WRITE_BYTE:
	movfw	My_EEADDR			; Load eeprom address which want to write
	bsf	STATUS, RP1			; Select Bank2
	bcf	STATUS, RP0
	movwf	EEADR
	bcf	STATUS, RP1			; Select Bank0
	movfw	My_EEDATA			; Load eeprom data which want to write
	bsf	STATUS, RP1			; Select Bank2
	movwf	EEDATA				; Load eeprom data which want to write
	bsf	STATUS, RP0			; Select Bank3
	bcf	EECON1, EEPGD			; tell PIC, we want to w/r DATA EEPROM
	bsf	EECON1, WREN	 		; set write enable bit
	bcf 	INTCON, GIE			; Disable all Interrupt before write EEPROM
	movlw	h'55'				; Send 55,AA sequence
	movwf	EECON2
	movlw	h'AA'
	movwf	EECON2
	bsf	EECON1,WR			; Begin write
	bsf	INTCON, GIE			; Enable all Interrupts again
	bcf	STATUS, RP1			; Return to Bank0 (default Bank)
	bcf	STATUS, RP0
Wait_WR:				
	btfss	PIR2,EEIF			; Wait until write successed flag setted
	goto	Wait_WR
	bcf	PIR2,EEIF			; Clear write successed flag
	bsf	STATUS, RP1			; Set to Bank2
	bsf	STATUS, RP0
	bcf	EECON1, WREN	 		; Disable EEPROM write to protect unusually write
	bcf	STATUS, RP1			; Return to Bank0 (default Bank)
	bcf	STATUS, RP0
	return
;-----------------------------------------------------------------------------
; EE_READ_BYTE - Routine to read a byte from EEPROM 
; DATA IN - My_EEDATA	<- data that read form EEPROM
;	  - My_EEADDR   <- address that want to read
;-----------------------------------------------------------------------------
EE_READ_BYTE:
	movfw	My_EEADDR			; Load eeprom address which want to read
	bsf	STATUS, RP1			; Select Bank2
	bcf	STATUS, RP0
	movwf	EEADR
	bsf	STATUS, RP0			; Select Bank3
	bcf	EECON1, EEPGD			; tell PIC, we want to w/r DATA EEPROM
	bsf	EECON1, RD			; Begin Read
Wait_RD:					; Wait until read successed flag setted
	btfsc	EECON1,RD
	goto	Wait_RD
	bcf	STATUS, RP0			; Select Bank2
	movfw	EEDATA				; Load EEPROM data
	bcf	STATUS, RP1			; Select Bank0
	movwf	My_EEDATA
	return
;-----------------------------------------------------------------------------
; Initialize Timer
;-----------------------------------------------------------------------------
Init_Timer:
	movlw	b'00111101'			; Fosc/4,Pre=1:8
	movwf	T1CON
        return
;-----------------------------------------------------------------------------
; Initialize Port
;-----------------------------------------------------------------------------
Init_Ports:
	bsf     STATUS,5			; Select bank 1
	bcf	STATUS,6
	bcf     OPTION_REG,5			; Internal intstruction cycle clock (CLKOUT)
	bcf	OPTION_REG,7			; Enable PORTB pull up
	movlw   b'10000111'			; RA0-RA5,RE0-RE2 as DIGITAL I/O
	movwf   ADCON1
;-----------------------------------------------------------------------------
; Port A setup
;-----------------------------------------------------------------------------
;RA0 (OUT1)		act as output
;RA1 (OUT2)		act as output
;RA2 (OUT3)		act as output 
;RA3 (OUT4)		act as output
;RA4 (OUT5) 		act as output
;RA5 (OUT6)		act as output
;-----------------------------------------------------------------------------
	movlw   0x0		
        movwf   TRISA           
;-----------------------------------------------------------------------------
; Port E setup
;-----------------------------------------------------------------------------
;RE0 (not use)		act as output
;RE1 (not use)		act as output
;RE2 (not use)		act as output
;-----------------------------------------------------------------------------
	movlw   b'00000000'
	movwf   TRISE
;-----------------------------------------------------------------------------
; Port B setup
;-----------------------------------------------------------------------------
;RB0 (not use)		act as output
;RB1 (not use)		act as output
;RB2 (not use)		act as output
;RB3 (not use)		act as output
;RB4 (not use)		act as output
;RB5 (not use)		act as output
;RB6 (not use)		act as output
;RB7 (not use)		act as output
;-----------------------------------------------------------------------------
	movlw	b'00000000'
	movwf	TRISB
;-----------------------------------------------------------------------------
; Port C setup
;-----------------------------------------------------------------------------
;RC0 (not use)		act as output
;RC1 (not use)		act as output
;RC2 (not use)		act as output
;RC3 (not use)		act as output
;RC4 (not use)		act as output
;RC5 (not use)		act as output
;RC6 (not use)		act as output
;RC7 (not use)		act as output
;-----------------------------------------------------------------------------
	movlw   b'00000000'
	movwf   TRISC
;-----------------------------------------------------------------------------
; Port D setup
;-----------------------------------------------------------------------------
;RD0 (not use)		act as output
;RD1 (not use)		act as output
;RD2 (not use)		act as output
;RD3 (not use)		act as output
;RD4 (not use)		act as output
;RD5 (not use)		act as output
;RD6 (not use)		act as output
;RD7 (not use)		act as output
;-----------------------------------------------------------------------------
	movlw   b'00000000'
	movwf   TRISD
;-----------------------------------------------------------------------------
	bcf     STATUS,5			; Select page 0
	bcf	STATUS,6
        return

	END                       ; directive 'end of program'

